@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2009-2019, Simon Kennedy, sffjunkie+code@gmail.com
+# Copyright 2009-2021, Simon Kennedy, sffjunkie+code@gmail.com
 
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -30,6 +30,7 @@ The :mod:`astral` package provides the means to calculate the following times of
 * blue hour
 * golden hour
 * rahukaalam
+* moon rise, set, azimuth and zenith
 
 plus solar azimuth and elevation at a specific latitude/longitude.
 It can also calculate the moon phase for a specific date.
@@ -38,24 +39,19 @@ The package also provides a self contained geocoder to turn a small set of
 location names into timezone, latitude and longitude. The lookups
 can be perfomed using the :func:`~astral.geocoder.lookup` function defined in
 :mod:`astral.geocoder`
-
-.. note::
-
-   The `Astral` and `GoogleGeocoder` classes from earlier versions have been
-   removed.
 """
 
 import datetime
 import re
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from enum import Enum
-from math import fabs
-from typing import Any, Tuple, Union
+from math import radians, tan
+from typing import Optional, Tuple, Union
 
 try:
-    import pytz
+    import zoneinfo
 except ImportError:
-    raise ImportError(("The astral module requires the pytz module to be available."))
+    from backports import zoneinfo
 
 
 __all__ = [
@@ -63,29 +59,41 @@ __all__ = [
     "SunDirection",
     "Observer",
     "LocationInfo",
+    "AstralBodyPosition",
     "now",
     "today",
     "dms_to_float",
+    "refraction_at_zenith",
 ]
 
-__version__ = "2.1"
+__version__ = "3.0"
 __author__ = "Simon Kennedy <sffjunkie+code@gmail.com>"
 
 
+TimePeriod = Tuple[datetime.datetime, datetime.datetime]
 Elevation = Union[float, Tuple[float, float]]
+Degrees = float
+Radians = float
+Minutes = float
 
 
-def now(tzinfo: datetime.tzinfo = pytz.utc) -> datetime.datetime:
+def now(tz: Optional[datetime.tzinfo] = None) -> datetime.datetime:
     """Returns the current time in the specified time zone"""
-    return pytz.utc.localize(datetime.datetime.utcnow()).astimezone(tzinfo)
+    now_utc = datetime.datetime.now(datetime.timezone.utc)
+    if tz is None:
+        return now_utc
+
+    return now_utc.astimezone(tz)
 
 
-def today(tzinfo: datetime.tzinfo = pytz.utc) -> datetime.date:
+def today(tz: Optional[datetime.tzinfo] = None) -> datetime.date:
     """Returns the current date in the specified time zone"""
-    return now(tzinfo).date()
+    return now(tz).date()
 
 
-def dms_to_float(dms: Union[str, float, Elevation], limit: float = -1) -> float:
+def dms_to_float(
+    dms: Union[str, float, Elevation], limit: Optional[float] = None
+) -> float:
     """Converts as string of the form `degrees°minutes'seconds"[N|S|E|W]`,
     or a float encoded as a string, to a float
 
@@ -101,15 +109,15 @@ def dms_to_float(dms: Union[str, float, Elevation], limit: float = -1) -> float:
     """
 
     try:
-        res = float(dms) # type: ignore
-    except (ValueError, TypeError):
-        _dms_re = r"(?P<deg>\d{1,3})[°]((?P<min>\d{1,2})[′'])?((?P<sec>\d{1,2})[″\"])?(?P<dir>[NSEW])?"
-        m = re.match(_dms_re, str(dms), flags=re.IGNORECASE)
-        if m:
-            deg = m.group("deg") or 0.0
-            min_ = m.group("min") or 0.0
-            sec = m.group("sec") or 0.0
-            dir_ = m.group("dir") or "E"
+        res = float(dms)  # type: ignore
+    except (ValueError, TypeError) as exc:
+        _dms_re = r"(?P<deg>\d{1,3})[°]((?P<min>\d{1,2})[′'])?((?P<sec>\d{1,2})[″\"])?(?P<dir>[NSEW])?"  # noqa
+        dms_match = re.match(_dms_re, str(dms), flags=re.IGNORECASE)
+        if dms_match:
+            deg = dms_match.group("deg") or 0.0
+            min_ = dms_match.group("min") or 0.0
+            sec = dms_match.group("sec") or 0.0
+            dir_ = dms_match.group("dir") or "E"
 
             res = float(deg)
             if min_:
@@ -120,22 +128,86 @@ def dms_to_float(dms: Union[str, float, Elevation], limit: float = -1) -> float:
             if dir_.upper() in ["S", "W"]:
                 res = -res
         else:
-            raise ValueError("Unable to convert degrees/minutes/seconds to float")
+            raise ValueError(
+                "Unable to convert degrees/minutes/seconds to float"
+            ) from exc
 
-    if res > limit:
-        res = limit
-    elif res < -limit:
-        res = -limit
+    if limit is not None:
+        if res > limit:
+            res = limit
+        elif res < -limit:
+            res = -limit
 
     return res
 
 
+def hours_to_time(value: float) -> datetime.time:
+    """Convert a floating point number of hours to a datetime.time"""
+
+    hour = int(value)
+    value -= hour
+    value *= 60
+    minute = int(value)
+    value -= minute
+    value *= 60
+    second = int(value)
+    value -= second
+    microsecond = int(value * 1000000)
+
+    return datetime.time(hour, minute, second, microsecond)
+
+
+def time_to_hours(value: datetime.time) -> float:
+    """Convert a datetime.time to a floating point number of hours"""
+
+    hours = 0.0
+    hours += value.hour
+    hours += value.minute / 60
+    hours += value.second / 3600
+    hours += value.microsecond / 1000000
+
+    return hours
+
+
+def time_to_seconds(value: datetime.time) -> float:
+    """Convert a datetime.time to a floating point number of seconds"""
+
+    hours = time_to_hours(value)
+    return hours * 3600
+
+
+def refraction_at_zenith(zenith: float) -> float:
+    """Calculate the degrees of refraction of the sun due to the sun's elevation."""
+
+    elevation = 90 - zenith
+    if elevation >= 85.0:
+        return 0
+
+    refraction_correction = 0.0
+    te = tan(radians(elevation))
+    if elevation > 5.0:
+        refraction_correction = (
+            58.1 / te - 0.07 / (te * te * te) + 0.000086 / (te * te * te * te * te)
+        )
+    elif elevation > -0.575:
+        step1 = -12.79 + elevation * 0.711
+        step2 = 103.4 + elevation * step1
+        step3 = -518.2 + elevation * step2
+        refraction_correction = 1735.0 + elevation * step3
+    else:
+        refraction_correction = -20.774 / te
+
+    refraction_correction = refraction_correction / 3600.0
+
+    return refraction_correction
+
+
 class Depression(Enum):
     """The depression angle in degrees for the dawn/dusk calculations"""
 
-    CIVIL: float = 6.0
-    NAUTICAL: float = 12.0
-    ASTRONOMICAL: float = 18.0
+    CIVIL = 6
+    NAUTICAL = 12
+    ASTRONOMICAL = 18
 
 
 class SunDirection(Enum):
@@ -145,6 +217,15 @@ class SunDirection(Enum):
     SETTING = -1
 
 
+@dataclass
+class AstralBodyPosition:
+    """The position of an astral body as seen from earth"""
+
+    right_ascension: Radians = field(default_factory=float)
+    declination: Radians = field(default_factory=float)
+    distance: Radians = field(default_factory=float)
+
+
 @dataclass
 class Observer:
     """Defines the location of an observer on Earth.
@@ -170,8 +251,8 @@ class Observer:
                     in metres above/below the location.
     """
 
-    latitude: float = 51.4733
-    longitude: float = -0.0008333
+    latitude: Degrees = 51.4733
+    longitude: Degrees = -0.0008333
     elevation: Elevation = 0.0
 
     def __setattr__(self, name: str, value: Union[str, float, Elevation]):
@@ -191,29 +272,29 @@ class Observer:
 class LocationInfo:
     """Defines a location on Earth.
 
-    Latitude and longitude can be set either as a float or as a string. For strings they must
-    be of the form
+    Latitude and longitude can be set either as a float or as a string.
+    For strings they must be of the form
 
         degrees°minutes'seconds"[N|S|E|W] e.g. 51°31'N
 
     `minutes’` & `seconds”` are optional.
 
     Args:
-        name:      Location name (can be any string)
-        region:    Region location is in (can be any string)
-        timezone:  The location's time zone (a list of time zone names can be obtained from
-                      `pytz.all_timezones`)
-        latitude:  Latitude - Northern latitudes should be positive
-        longitude: Longitude - Eastern longitudes should be positive
+        name:       Location name (can be any string)
+        region:     Region location is in (can be any string)
+        timezone:   The location's time zone (a list of time zone names can be
+                    obtained from `zoneinfo.available_timezones`)
+        latitude:   Latitude - Northern latitudes should be positive
+        longitude:  Longitude - Eastern longitudes should be positive
     """
 
     name: str = "Greenwich"
     region: str = "England"
     timezone: str = "Europe/London"
-    latitude: float = 51.4733
-    longitude: float = -0.0008333
+    latitude: Degrees = 51.4733
+    longitude: Degrees = -0.0008333
 
-    def __setattr__(self, name: str, value: Union[float, str]):
+    def __setattr__(self, name: str, value: Union[Degrees, str]):
         if name == "latitude":
             value = dms_to_float(value, 90.0)
         elif name == "longitude":
@@ -225,6 +306,12 @@ class LocationInfo:
         """Return an Observer at this location"""
         return Observer(self.latitude, self.longitude, 0.0)
 
+    @property
+    def tzinfo(self):  # type: ignore
+        """Return a zoneinfo.ZoneInfo for this location"""
+        return zoneinfo.ZoneInfo(self.timezone)  # type: ignore
+
     @property
     def timezone_group(self):
-        return self.timezone.split("/")[0]
+        """Return the group a timezone is in"""
+        return self.timezone.split("/", maxsplit=1)[0]
